@@ -49,9 +49,11 @@ from sphinx.util.websupport import is_commentable
 from sphinx.errors import SphinxError, ExtensionError
 from sphinx.locale import _
 from sphinx.versioning import add_uids, merge_doctrees
-from sphinx.transforms import DefaultSubstitutions, MoveModuleTargets, \
-    HandleCodeBlocks, AutoNumbering, SortIds, CitationReferences, Locale, \
-    RemoveTranslatableInline, SphinxContentsFilter, ExtraTranslatableNodes
+from sphinx.transforms import (
+    DefaultSubstitutions, MoveModuleTargets, ApplySourceWorkaround,
+    HandleCodeBlocks, AutoNumbering, SortIds, CitationReferences, Locale,
+    RemoveTranslatableInline, SphinxContentsFilter, ExtraTranslatableNodes,
+)
 
 
 orig_role_function = roles.role
@@ -99,7 +101,7 @@ class SphinxStandaloneReader(standalone.Reader):
     """
     Add our own transforms.
     """
-    transforms = [ExtraTranslatableNodes, Locale, CitationReferences,
+    transforms = [ApplySourceWorkaround, ExtraTranslatableNodes, Locale, CitationReferences,
                   DefaultSubstitutions, MoveModuleTargets, HandleCodeBlocks,
                   AutoNumbering, SortIds, RemoveTranslatableInline]
 
@@ -141,8 +143,6 @@ class SphinxFileInput(FileInput):
     def __init__(self, app, env, *args, **kwds):
         self.app = app
         self.env = env
-        # don't call sys.exit() on IOErrors
-        kwds['handle_io_errors'] = False
         kwds['error_handler'] = 'sphinx'  # py3: handle error on open.
         FileInput.__init__(self, *args, **kwds)
 
