@@ -14,15 +14,17 @@
        generate:
                sphinx-autogen2 -o source/generated source/*.rst
 
-    :copyright: Copyright 2007-2014 by the Sphinx team, see AUTHORS.
+    :copyright: Copyright 2007-2015 by the Sphinx team, see AUTHORS.
     :license: BSD, see LICENSE for details.
 """
+from __future__ import print_function
 
 import os
 import re
 import sys
 import pydoc
 import optparse
+import codecs
 
 from jinja2 import FileSystemLoader, TemplateNotFound
 from jinja2.sandbox import SandboxedEnvironment
@@ -69,11 +71,14 @@ def main(argv=sys.argv):
                               "." + options.suffix,
                               template_dir=options.templates)
 
+
 def _simple_info(msg):
-    print msg
+    print(msg)
+
 
 def _simple_warn(msg):
-    print >> sys.stderr, 'WARNING: ' + msg
+    print('WARNING: ' + msg, file=sys.stderr)
+
 
 # -- Generating output ---------------------------------------------------------
 
@@ -109,14 +114,11 @@ def generate_autosummary_docs(sources, output_dir=None, suffix='.rst',
     # read
     items = find_autosummary_in_files(sources)
 
-    # remove possible duplicates
-    items = dict([(item, True) for item in items]).keys()
-
     # keep track of new files
     new_files = []
 
     # write
-    for name, path, template_name in sorted(items, key=str):
+    for name, path, template_name in sorted(set(items), key=str):
         if path is None:
             # The corresponding autosummary:: directive did not have
             # a :toctree: option
@@ -126,8 +128,8 @@ def generate_autosummary_docs(sources, output_dir=None, suffix='.rst',
         ensuredir(path)
 
         try:
-            name, obj, parent = import_by_name(name)
-        except ImportError, e:
+            name, obj, parent, mod_name = import_by_name(name)
+        except ImportError as e:
             warn('[autosummary] failed to import %r: %s' % (name, e))
             continue
 
@@ -139,9 +141,7 @@ def generate_autosummary_docs(sources, output_dir=None, suffix='.rst',
 
         new_files.append(fn)
 
-        f = open(fn, 'w')
-
-        try:
+        with open(fn, 'w') as f:
             doc = get_documenter(obj, parent)
 
             if template_name is not None:
@@ -172,17 +172,17 @@ def generate_autosummary_docs(sources, output_dir=None, suffix='.rst',
             if doc.objtype == 'module':
                 ns['members'] = dir(obj)
                 ns['functions'], ns['all_functions'] = \
-                                   get_members(obj, 'function')
+                    get_members(obj, 'function')
                 ns['classes'], ns['all_classes'] = \
-                                 get_members(obj, 'class')
+                    get_members(obj, 'class')
                 ns['exceptions'], ns['all_exceptions'] = \
-                                   get_members(obj, 'exception')
+                    get_members(obj, 'exception')
             elif doc.objtype == 'class':
                 ns['members'] = dir(obj)
                 ns['methods'], ns['all_methods'] = \
-                                 get_members(obj, 'method', ['__init__'])
+                    get_members(obj, 'method', ['__init__'])
                 ns['attributes'], ns['all_attributes'] = \
-                                 get_members(obj, 'attribute')
+                    get_members(obj, 'attribute')
 
             parts = name.split('.')
             if doc.objtype in ('method', 'attribute'):
@@ -203,8 +203,6 @@ def generate_autosummary_docs(sources, output_dir=None, suffix='.rst',
 
             rendered = template.render(**ns)
             f.write(rendered)
-        finally:
-            f.close()
 
     # descend recursively to new files
     if new_files:
@@ -223,27 +221,33 @@ def find_autosummary_in_files(filenames):
     """
     documented = []
     for filename in filenames:
-        f = open(filename, 'r')
-        lines = f.read().splitlines()
-        documented.extend(find_autosummary_in_lines(lines, filename=filename))
-        f.close()
+        with codecs.open(filename, 'r', encoding='utf-8',
+                         errors='ignore') as f:
+            lines = f.read().splitlines()
+            documented.extend(find_autosummary_in_lines(lines,
+                                                        filename=filename))
     return documented
 
+
 def find_autosummary_in_docstring(name, module=None, filename=None):
     """Find out what items are documented in the given object's docstring.
 
     See `find_autosummary_in_lines`.
     """
     try:
-        real_name, obj, parent = import_by_name(name)
+        real_name, obj, parent, modname = import_by_name(name)
         lines = pydoc.getdoc(obj).splitlines()
         return find_autosummary_in_lines(lines, module=name, filename=filename)
     except AttributeError:
         pass
-    except ImportError, e:
-        print "Failed to import '%s': %s" % (name, e)
+    except ImportError as e:
+        print("Failed to import '%s': %s" % (name, e))
+    except SystemExit as e:
+        print("Failed to import '%s'; the module executes module level "
+              "statement and it might call sys.exit()." % name)
     return []
 
+
 def find_autosummary_in_lines(lines, module=None, filename=None):
     """Find out what items appear in autosummary:: directives in the
     given lines.
@@ -288,7 +292,7 @@ def find_autosummary_in_lines(lines, module=None, filename=None):
                 continue
 
             if line.strip().startswith(':'):
-                continue # skip options
+                continue  # skip options
 
             m = autosummary_item_re.match(line)
             if m:
@@ -296,7 +300,7 @@ def find_autosummary_in_lines(lines, module=None, filename=None):
                 if name.startswith('~'):
                     name = name[1:]
                 if current_module and \
-                       not name.startswith(current_module + '.'):
+                   not name.startswith(current_module + '.'):
                     name = "%s.%s" % (current_module, name)
                 documented.append((name, toctree, template))
                 continue
